"use strict";
/**
 * Copyright (c) 2018-2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.StructureRepresentationRegistry = void 0;
var tslib_1 = require("tslib");
var object_1 = require("../../mol-util/object");
var representation_1 = require("../representation");
var ball_and_stick_1 = require("./representation/ball-and-stick");
var carbohydrate_1 = require("./representation/carbohydrate");
var cartoon_1 = require("./representation/cartoon");
var ellipsoid_1 = require("./representation/ellipsoid");
var gaussian_surface_1 = require("./representation/gaussian-surface");
var label_1 = require("./representation/label");
var molecular_surface_1 = require("./representation/molecular-surface");
var orientation_1 = require("./representation/orientation");
var point_1 = require("./representation/point");
var putty_1 = require("./representation/putty");
var spacefill_1 = require("./representation/spacefill");
var line_1 = require("./representation/line");
var gaussian_volume_1 = require("./representation/gaussian-volume");
var backbone_1 = require("./representation/backbone");
var StructureRepresentationRegistry = /** @class */ (function (_super) {
    tslib_1.__extends(StructureRepresentationRegistry, _super);
    function StructureRepresentationRegistry() {
        var _this = _super.call(this) || this;
        object_1.objectForEach(StructureRepresentationRegistry.BuiltIn, function (p, k) {
            if (p.name !== k)
                throw new Error("Fix BuiltInStructureRepresentations to have matching names. " + p.name + " " + k);
            _this.add(p);
        });
        return _this;
    }
    return StructureRepresentationRegistry;
}(representation_1.RepresentationRegistry));
exports.StructureRepresentationRegistry = StructureRepresentationRegistry;
(function (StructureRepresentationRegistry) {
    StructureRepresentationRegistry.BuiltIn = {
        'cartoon': cartoon_1.CartoonRepresentationProvider,
        'backbone': backbone_1.BackboneRepresentationProvider,
        'ball-and-stick': ball_and_stick_1.BallAndStickRepresentationProvider,
        'carbohydrate': carbohydrate_1.CarbohydrateRepresentationProvider,
        'ellipsoid': ellipsoid_1.EllipsoidRepresentationProvider,
        'gaussian-surface': gaussian_surface_1.GaussianSurfaceRepresentationProvider,
        'gaussian-volume': gaussian_volume_1.GaussianVolumeRepresentationProvider,
        'label': label_1.LabelRepresentationProvider,
        'line': line_1.LineRepresentationProvider,
        'molecular-surface': molecular_surface_1.MolecularSurfaceRepresentationProvider,
        'orientation': orientation_1.OrientationRepresentationProvider,
        'point': point_1.PointRepresentationProvider,
        'putty': putty_1.PuttyRepresentationProvider,
        'spacefill': spacefill_1.SpacefillRepresentationProvider,
    };
})(StructureRepresentationRegistry = exports.StructureRepresentationRegistry || (exports.StructureRepresentationRegistry = {}));
exports.StructureRepresentationRegistry = StructureRepresentationRegistry;
//# sourceMappingURL=registry.js.map