/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { StructureRepresentation, StructureRepresentationProvider } from '../representation';
import { RepresentationParamsGetter, RepresentationContext } from '../../../mol-repr/representation';
import { Structure } from '../../../mol-model/structure';
import { ThemeRegistryContext } from '../../../mol-theme/theme';
export declare const PuttyParams: {
    sizeFactor: PD.Numeric;
    visuals: PD.MultiSelect<"polymer-tube" | "polymer-gap">;
    radialSegments: PD.Numeric;
    unitKinds: PD.MultiSelect<"atomic" | "spheres" | "gaussians">;
    includeParent: PD.BooleanParam;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    detail: PD.Numeric;
    linearSegments: PD.Numeric;
};
export declare type PuttyParams = typeof PuttyParams;
export declare function getPuttyParams(ctx: ThemeRegistryContext, structure: Structure): {
    sizeFactor: PD.Numeric;
    visuals: PD.MultiSelect<"polymer-tube" | "polymer-gap">;
    radialSegments: PD.Numeric;
    unitKinds: PD.MultiSelect<"atomic" | "spheres" | "gaussians">;
    includeParent: PD.BooleanParam;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    detail: PD.Numeric;
    linearSegments: PD.Numeric;
};
export declare type PuttyRepresentation = StructureRepresentation<PuttyParams>;
export declare function PuttyRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<Structure, PuttyParams>): PuttyRepresentation;
export declare const PuttyRepresentationProvider: StructureRepresentationProvider<{
    sizeFactor: PD.Numeric;
    visuals: PD.MultiSelect<"polymer-tube" | "polymer-gap">;
    radialSegments: PD.Numeric;
    unitKinds: PD.MultiSelect<"atomic" | "spheres" | "gaussians">;
    includeParent: PD.BooleanParam;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    detail: PD.Numeric;
    linearSegments: PD.Numeric;
}, "putty">;
